function htmltableToggleDiv(myId){
	// Create the selectors
	var tbody = "#"+myId+" div.body";	
	var hideButton = "#"+myId+" #hidecontrol";
	var showButton = "#"+myId+" #showcontrol";
	
	// Verify display property (CSS) of tbody
	var display = $(tbody).css("display");

	if(display=="none"){
		$(tbody).show("slide",{ direction: "up" }, 200);
		$(hideButton).show();
		$(showButton).hide();
	}else{
		$(tbody).hide("slide",{ direction: "up" }, 200);
		$(hideButton).hide();
		$(showButton).show();
	}
}

function htmltableToggleMode(myId){
	var buttonSelector = "#"+myId+" div.header table.header-table tr td.header-mode div.header-mode-button";
	// If in edit mode, change the rows
	if($(buttonSelector).hasClass("editmode")){
		htmltableView(myId);
	}else{
		htmltableEdit(myId);
	}
}

function htmltableEdit(myId){
	var buttonSelector = "#"+myId+" div.header table.header-table tr td.header-mode div.header-mode-button";
	var rowSelector = "#"+myId+" div.body table tbody tr";
	// Enter Edit Mode
	$(rowSelector).addClass("editrow");
	$(buttonSelector).addClass("editmode");
}

function htmltableView(myId){
	var buttonSelector = "#"+myId+" div.header table.header-table tr td.header-mode div.header-mode-button";
	var rowSelector = "#"+myId+" div.body table tbody tr";
	var formSelector = "#"+myId+" div.form";

	$(rowSelector).removeClass("editrow");
	$(buttonSelector).removeClass("editmode");
	$(formSelector).hide();
}


/*
 * Function to return the 
 * column headers.
 * Returns an array with all the 
 * column headers.
 */
function htmltableGetColumns(myId){
	// Get the column's headers
	var columnsSelector = "#"+myId+" div.body table.body-table tr.body-header th div.column-name span"
	var columns = new Array();
	$(columnsSelector).each(function(index){
		columns[index] = $(this).text();
	});	
	return columns;
}

/*
 * Global Variables
 * 
 * These are variables needed to store globally
 * some important parameters such as:
 * htmltable_globalId - This is the id of the widget been
 * used.
 * htmltable_globalRow - This is the row being clicked
 * html_globalFormData - Data extracted from the form.
 */
//Global Variables needed for row manipulation
var htmltable_globalId=null;
var htmltable_globalRow=null;
var htmltable_globalFormData=null;

/*
 * htmltableShowForm
 *  
 * This function is used to show the
 * edit form when a row is clicked. It has the following 
 * arguments:
 * myId - This is the Id of the particular widget that is being manipulated
 * myRow - This is the element that was clicked
 * formTitle - The title to be shown in the form.
 */
function htmltableShowForm(myId,myRow,formTitle){
	// Widget Reference
	var widgetSelector = "#"+myId;
	var divSelector = "#table-edit";//+myId+" div.form";
	var formSelector = "#table-form";//+myId+"-form";
	var rowSelector = "#"+myId+" div.body table tbody tr";
	
	// Close and Destroy form if already
	// opened.
	htmltableCloseForm();
	
	// Check if row is in edit mode
	if($(rowSelector).hasClass("editrow")){
		// Set the global variable with the clicked row
		htmltable_globalId = myId;
		htmltable_globalRow = myRow;

		// Set Form Title
		$(divSelector+" p").text(formTitle);
		
		// Show the form
		var widgetLeft = $(widgetSelector).offset().left;
		var widgetTop = $(widgetSelector).position().top;
		var widgetWidth = $(widgetSelector).outerWidth();
		var widgetHeight = $(widgetSelector).outerHeight();
		$(divSelector).css('top',widgetTop);
		$(divSelector).css('left',widgetLeft+widgetWidth/4);
		$(divSelector).show("slide",{ direction: "up" }, 100);
		
		var columns = htmltableGetColumns(myId);
		
		// Get the html elements of the row that was clicked
		// and create the form
		var myString = "";
		$(myRow).children().each(function(index) {
			var openDiv = "<div class='edit-table-row'><table><tbody><tr>";
			var label = "<td><label for='"+columns[index]+"'>"+columns[index]+"</label></td>";
			var input = "<td><input type='text' name='"+columns[index]+"' class='text' value='"+$(this).text()+"'></input></td>";
			myString = myString + openDiv + label + input + "</tr></tbody></table></div>";
		});
		var changeButton = "<button type='button' onclick='htmltableModifyRow(htmltable_globalRow);'>Change</button>";
		var closeButton = "<button type='button' onclick='htmltableCloseForm();'>Close</button>";
		var submitButton = "<input type='submit' value='submit' class='button'></input>";
		$(formSelector).append(myString+changeButton+closeButton);

		var position = $(rowSelector).offset();
		$(divSelector).offset(position);
	}
}

/*
 * htmltableModifyRow
 * 
 * This function changes the table row
 * using the data in the form.
 * It uses one parameter:
 * myRow - Object representing the tr element
 * that was clicked.
 */
function htmltableModifyRow(myRow){
	htmltable_globalFormData = new Array();
	
	// Get all the form entries
	$("#table-form input.text").each(function(index){
		htmltable_globalFormData[index] = $(this).val();
	});
	
	// Write to the table
	$(myRow).children().each(function(index) {
		$(this).text(htmltable_globalFormData[index]);
	});
	$(myRow).addClass("modified");
	
	htmltable_globalFormData=null;
	
	// Show send button
	var sendButton = "#"+htmltable_globalId+" div.header table.header-table tr td.header-mode div.header-send-button";
	$(sendButton).show();
	
	//Update Table Sorter Plugin
	$("#"+htmltable_globalId+"-body-table").trigger("update");
	
}

/*
 * htmltableCloseForm
 * 
 * This function cleanups the widget
 * First it hides the Form's div container
 * then it removes all form elements from the form
 * Then it resets all the global variables
 */
function htmltableCloseForm(){
	$("#table-edit").hide();	
	$("#table-form").children().remove();
	htmltable_globalId=null;
	htmltable_globalRow=null;
	htmltable_globalFormData=null;
}



/*
 * htmltableSend
 * 
 * This function sends the XHTTPRequest
 * through AJAX. It uses three parameters:
 * myId - the widget id
 * url - url for the request in a Controller/Action format
 * target - an element where the answer of the request should be
 * directed to
 */
function htmltableSend(myId,url,target){
	var rowSelector = "#"+myId+" div.body table tbody tr";
	var modifiedSelector = "#"+myId+" div.body table tbody tr.modified";
	var rows = new Object();
	var columns = htmltableGetColumns(myId);
	
	//<---- GET ALL MODIFIED ROWS ---->
	// For each modified row....
	$.each($(modifiedSelector),function(key,value){
		var rowKey = "htmltable-"+myId+"-row-"+key;
		var myElements = new Object();
		$(value).children().each(function(index){
			//... get its cell's values
			myElements[columns[index]] = $(this).text();
		});
		//...and add them as key-value pairs
		//...in an object called rows
		rows[rowKey] = myElements;
		myElements = null;
	});
	// The resulting object or array
	//... will be send as an http request
	//... through AJAX
	
	//<---- SEND XHR ----->
	var jqXHR = $.ajax({
		url: "index.php?r="+url,
		type: "POST",
		data: $.param(rows),
		beforeSend: function(xhr,settings){
			$("#"+myId+"-send-button")
				.addClass("ajaxLoading");
			//alert(decodeURIComponent($.param(rows)));
		},
		success: function(returnedData,status,xhr){
			alert("Success! server says: "+returnedData);
			//$(target).text(returnedData);
		},
		error: function(xhr,errorText,errorObj){
			alert(errorText);
		},
		complete: function(xhr,status){
			$(rowSelector).removeClass("modified");
			$("#"+myId+"-send-button")
				.removeClass("ajaxLoading")
				.hide();
		},
	});
	
	htmltableView(myId);
	htmltableCloseForm();
}

/*
 * htmltableSort
 * 
 * This function enables the TableSorter 2.0
 * jQuery plugin on the table for client sided
 * sorting. It has three parameters:
 * myId - the widget's id
 * sortColumn - an index starting from 0 referring
 * to the column to use for sorting initially
 * sortOrder - 0-ASC, 1-DESC
 * 
 */
function htmltableSort(myId,sortColumn,sortOrder){
	// Enable table sorter for the table
	var selector = '#'+myId+'-body-table';
	
	$(selector).tablesorter({
		cssHeader: "unsorted",
		sortList: [[sortColumn,sortOrder]],
	});

	// Set the cursor to a hand or pointer over the
	// cells used to sort.
	selector = selector + ' thead tr.body-header th';
	$(selector).css("cursor","pointer");
}

function htmltablePager(myId,size){
	// Enable table sorter for the table
	var selector = '#'+myId+'-body-table';
	
	$(selector).tablesorterPager({
		container:$("#"+myId+"-pager"),
		size:size,
	});	
}
